/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

/**
 * Remote interface for the CancelEJB. The remote interface defines all possible
 * business methods for the bean. These are the methods going tobe invoked remotely
 * by clients, once they have a reference to the remote interface.
 *
 * Clients (MovieServlet, in this case), generally take the help of JNDI to lookup
 * the bean's home interface (CancelHome, in this case) and then use the home interface
 * to obtain references to the bean's remote interface (Cancel, in this case).
 *
 */
public interface Cancel extends javax.ejb.EJBObject
{
	/**
	 *	Initializes Cancel class to store and process
	 *	purchase information from form.
	 *
	 *	@throws java.rmi.RemoteException
	 */
    public void Init ( ) throws java.rmi.RemoteException;

	/**
	 *	Initializes parameters from forms (via request).
	 *
	 *	@param	params Values to assign to needed parameters
	 *	@throws java.rmi.RemoteException
	 */
    public void InitParams ( String[] params ) throws java.rmi.RemoteException;

	/**
	 *	Initializes needed session attributes.
	 *
	 *	@param	sattribs Values to assign to needed parameters
	 *	@throws java.rmi.RemoteException
	 */
    public void InitSAttribs ( String[] sattribs ) throws java.rmi.RemoteException;

	/**
	 *	Processes stored purchase data.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Contents to display to interface
	 */
    public boolean process ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns error state
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Possible errors encountered
	 */
	public String getError ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns page to go to
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Page to go to after processing
	 */
	public String getGoto ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from request form fields.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of parameter names
	 */
	public String[] getParameters ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from the session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of session names to set
	 */
	public String[] getSAttributes ( ) throws java.rmi.RemoteException;
}
