/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

import java.util.*;
import java.io.*;
import javax.naming.*;
import java.sql.*;
import javax.sql.*;

import edu.cmu.fantango.util.*;

/**
 * A simple stateless bean for the Login application. This bean implements one
 * business method as declared by the remote interface.
 */
public class LoginEJB implements javax.ejb.SessionBean
{
    private  javax.ejb.SessionContext m_ctx = null;

    /**
     *	Stored username from form
     */
	private String s_Username;

    /**
     *	Stored password from form
     */
	private String s_Password;

    /**
     *	Stored user logged in
     */
	private String s_CurrentUser;

    /**
     *	Parameters that the bean needs from page forms
     */
	private String[] s_Parameters;

    /**
     *	Parameters that the bean needs from the session
     */
	private String[] s_SAttributes;

    /**
     *	Store error
     */
	private String s_Error;

    /**
     *	Store current goto page
     */
	private String s_Goto;

    /**
     *	Page to load on success
     */
	private static final String s_GotoSuccess = "/index.jsp";

    /**
     *	Page to load on failure
     */
	private static final String s_GotoFailure = "/Login.jsp";

    /**
     *	Table to draw queries from
     */
	private static final String s_Table = "FT_CUSTOMER";

    /**
     * Required by EJB spec.
     */
    public void Login ( )
    {

    }


	/**
	 *	Initializes Login class to store and process
	 *	login information from form.
	 *
	 *	@throws java.rmi.RemoteException
	 */
    public void Init ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::LoginEJB:: (Init)" );
		s_Parameters = new String[2];
		s_Parameters[0] = "username";
		s_Parameters[1] = "password";
		s_SAttributes = new String[1];
		s_SAttributes[0] = "userloggedin";
		s_Goto = s_GotoSuccess;
		s_Error = "&nbsp;";
    }

	/**
	 *	Initializes parameters from forms (via request)
	 *
	 *	@param	params Values to assign to needed parameters
	 *	@throws java.rmi.RemoteException
	 */
    public void InitParams ( String[] params ) throws java.rmi.RemoteException
    {
		System.out.println ( "::LoginEJB:: (InitParams)" );
		s_Username = params[0];
		s_Password = params[1];
    }


	/**
	 *	Initializes attributes from session
	 *
	 *	@param	sattribs Values to assign to needed parameters
	 *	@throws java.rmi.RemoteException
	 */
    public void InitSAttribs ( String[] sattribs ) throws java.rmi.RemoteException
    {
		System.out.println ( "::LoginEJB:: (InitsSAttribs)" );
		s_CurrentUser = sattribs[0];
	}


	/**
	 *	Composes string for SQL select statement
	 *
	 *	@returns SQL select statement to check username
	 */
	private String getUsernameCheck ( )
	{
		System.out.println ( "::LoginEJB:: (getUsernameCheck)" );
		return "SELECT * FROM FanTango." + s_Table + " WHERE " +
				"Username = \"" + s_Username + "\"";
	}


	/**
	 *	Composes string for SQL select statement
	 *
	 *	@returns SQL select statement to check password
	 */
	private String getPasswordCheck ( )
	{
		System.out.println ( "::LoginEJB:: (getPasswordCheck)" );
		return this.getUsernameCheck ( ) +
				"AND Password = \"" + s_Password + "\"";

	}


	/**
	 *	Returns error state
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Possible errors encountered
	 */
	public String getError ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::LoginEJB:: (getError) " + s_Error );
		return s_Error;
	}


	/**
	 *	Returns page to go to
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Page to go to after processing
	 */
	public String getGoto ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::LoginEJB:: (getGoto) " + s_Goto );
		return s_Goto;
	}


	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from request form fields.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of parameter names
	 */
	public String[] getParameters ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::LoginEJB:: (getParameters)" );
		return s_Parameters;
	}


	/**
	 *	Returns array of String attributes that need to be set for the
	 *	session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of session attributes to set
	 */
	public String[] getSessionAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::LoginEJB:: (getSessionAttributes)" );
		String[] retSession = new String[2];
		retSession[0] = "userloggedin";
		retSession[1] = s_Username;

		return retSession;
	}


	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from the session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of parameter names
	 */
	public String[] getSAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::LoginEJB:: (getSAttributes)" );
		return s_SAttributes;
	}


	/**
	 *	Processes stored login data
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Success or Failure of the operation
	 */
    public boolean process ( ) throws java.rmi.RemoteException
    {
		try
		{
			System.out.println ( "::LoginEJB:: (process) Begin." );
			Connection conn = null;
			Statement stmt = null;
			ResultSet result = null;
			ResourceBundle rb = ResourceBundle.getBundle ( "LocalStrings", Locale.getDefault ( ) );

			if ( s_CurrentUser != null )
			{
				System.out.println ( "::LoginEJB:: (process) Already logged in." );
				s_Goto = s_GotoFailure;
				s_Error = rb.getString ( "invalid_userlogged" );
				return false;
			}

			System.out.println ( "::LoginEJB:: (process) Starting Datasource access." );

			try
			{
				conn = DBUtil.getDBConnection ( );
				stmt = conn.createStatement ( );
				result = stmt.executeQuery ( this.getUsernameCheck ( ) );
				System.out.println ( "::LoginEJB:: (process) Username query executed." );

				// Username exists
				if ( result.next ( ) )
				{
					System.out.println ( "::LoginEJB:: (process) Username exists." );
					result = stmt.executeQuery ( this.getPasswordCheck ( ) );
					System.out.println ( "::LoginEJB:: (process) Password query executed." );

					// Username and pass exist
					if ( result.next ( ) )
					{
						System.out.println ( "::LoginEJB:: (process) Password correct." );
						s_Goto = s_GotoSuccess;
						return true;
					}
					else
					{
						System.out.println ( "::LoginEJB:: (process) Password incorrect." );
						s_Goto = s_GotoFailure;
						s_Error = rb.getString ( "invalid_passl" );
						return false;
					}
				}
				// Username doesn't exist
				else
				{
					System.out.println ( "::LoginEJB:: (process) Username does not exist." );
					s_Goto = s_GotoFailure;
					s_Error = rb.getString ( "invalid_userl" );
					return false;
				}

			}
			catch ( Exception e )
			{
				//e.printStackTrace ( );

				System.out.println ( "::LoginEJB:: (process) Undefined error." );
				s_Goto = "/error.jsp";
				s_Error = "We are sorry, but we have encountered an undefined error.";
				return false;
			}
			finally
			{
				DBUtil.closeConnection ( conn );
				DBUtil.closeStatement ( stmt );
				DBUtil.closeResultSet ( result );
			}
		}
		catch ( Exception e )
		{
			System.out.println ( "::LoginEJB:: (process) Local Strings Undefined error." );
			s_Goto = "/error.jsp";
			s_Error = "We are sorry, but we have encountered an undefined error.";
			return false;
		}
	}

    /**
     * Sets the session context. Required by EJB spec.
     * @param ctx A SessionContext object.
     */
    public void setSessionContext(javax.ejb.SessionContext ctx) {
        m_ctx = ctx;
    }

    /**
     * Creates a bean. Required by EJB spec.
     * @exception throws CreateException.
     */
    public void ejbCreate() throws java.rmi.RemoteException, javax.ejb.CreateException {
        System.out.println("ejbCreate() on obj " + this);
    }

    /**
     * Removes the bean. Required by EJB spec.
     */
    public void ejbRemove() {
        System.out.println("ejbRemove() on obj " + this);
    }

    /**
     * Loads the state of the bean from secondary storage. Required by EJB spec.
     */
    public void ejbActivate() {
        System.out.println("ejbActivate() on obj " + this);
    }

    /**
     * Serializes the state of the bean to secondary storage. Required by EJB spec.
     */
    public void ejbPassivate() {
        System.out.println("ejbPassivate() on obj " + this);
    }

}
