/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

import java.util.*;
import java.io.*;
import javax.naming.*;
import java.sql.*;
import javax.sql.*;

import edu.cmu.fantango.util.*;

/**
 * A simple stateless bean for the Movie application. This bean implements one
 * business method as declared by the remote interface.
 */
public class PurchaseEJB implements javax.ejb.SessionBean
{
    private  javax.ejb.SessionContext m_ctx = null;

    /**
     *	Stored username from form
     */
	private String s_Username;

    /**
     *	Store showing id
     */
	private String s_ShowingID;

    /**
     *	Store ticket number
     */
	private String s_Tickets;

    /**
     *	Parameters the bean needs
     */
	private String[] s_Parameters;

    /**
     *	Parameters the bean needs from session attributes
     */
	private String[] s_sAttributes;

    /**
     *	Store error
     */
	private String s_Error;

    /**
     *	Store showing row
     */
	private String s_ShowingRow;

    /**
     *	Store current goto page
     */
	private static final String s_Goto = "/Confirm.jsp";

    /**
     *	Store table
     */
	private static final String s_OrderTable = "FT_ORDER";

    /**
     *	Store table
     */
	private static final String s_ShowingTable = "FT_SHOWING";

    /**
     * Required by EJB spec.
     */
    public void Purchase ( ) {
    }


	/**
	 *	Initializes Purchase class to store and process
	 *	purchase information from form.
	 *
	 *	@throws java.rmi.RemoteException
	 */
    public void Init ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::PurchaseEJB:: (Init)" );
		s_Parameters = new String[6];
		s_Parameters[0] = "showing";
		s_Parameters[1] = "tickets";
		s_Parameters[2] = "cctype";
		s_Parameters[3] = "ccnum";
		s_Parameters[4] = "ccmonth";
		s_Parameters[5] = "ccyear";
		s_sAttributes = new String[1];
		s_sAttributes[0] = "userloggedin";
		s_Error = "&nbsp;";
    }

	/**
	 *	Initializes Purchase class to store and process
	 *	register information from form.
	 *
	 *	@exception Throws java.rmi.RemoteException
	 */
    public void InitParams ( String[] params ) throws java.rmi.RemoteException
    {
		System.out.println ( "::PurchaseEJB:: (InitParams)" );
		s_ShowingID = params[0];
		s_Tickets = params[1];
    }


	/**
	 *	Initializes needed session attributes.
	 *
	 *	@param	sattribs Values to assign to needed parameters
	 *	@throws java.rmi.RemoteException
	 */
    public void InitSAttribs ( String[] sattribs ) throws java.rmi.RemoteException
    {
		System.out.println ( "::PurchaseEJB:: (InitSAttribs)" );
		s_Username = sattribs[0];
    }


	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from request form fields.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of parameter names
	 */
	public String[] getParameters ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::PurchaseEJB:: (getParameters)" );
		return s_Parameters;
	}


	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from the session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of session names to set
	 */
	public String[] getSAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::PurchaseEJB:: (getSAttributes)" );
		return s_sAttributes;
	}


	/**
	 *	Composes string for SQL order query statement.
	 *
	 *	@returns SQL insert statement to query order
	 */
	private String getOrderQuery ( )
	{
		System.out.println ( "::PurchaseEJB:: (getOrderQuery)" );
		return "SELECT * FROM FanTango." + s_OrderTable;
	}

	/**
	 *	Composes string for SQL order insert statement.
	 *
	 *	@returns SQL insert statement to insert into order
	 */
	private String getOrderInsert ( String into )
	{
		System.out.println ( "::PurchaseEJB:: (getOrderInsert)" );
		return "INSERT INTO FanTango." + s_OrderTable + " VALUES ( " +
				"'" + into + "', " +
				"'" + s_Username + "', " +
				"now(), " +
				"now(), " +
				"'" + s_ShowingID + "', " +
				"'" + s_Tickets + "', " +
				"0" +
				" )";
	}

	/**
	 *	Composes string for SQL showing update statement.
	 *
	 *	@returns SQL insert statement to update showing table
	 */
	private String getShowingUpdate ( )
	{
		System.out.println ( "::PurchaseEJB:: (getShowingUpdate)" );
		return "UPDATE FanTango." + s_ShowingTable +
				" SET Sold = Sold + " + s_Tickets + " WHERE " +
				" ID = " + s_ShowingID;
	}

	/**
	 *	Composes string for SQL showing query statement
	 *
	 *	@returns SQL insert statement to query showing table
	 */
	private String getShowingQuery ( )
	{
		System.out.println ( "::PurchaseEJB:: (getShowingQuery)" );
        return "SELECT t.Name, r.Name, m.Title, m.Length, s.Date, s.Time FROM " +
        		"FT_SHOWING s, FT_THEATER t, FT_ROOM r, FT_MOVIE m WHERE " +
        		"s.RoomID = r.ID AND " +
        		"s.MovieID = m.ID AND " +
        		"r.TheaterID = t.ID AND " +
        		"s.ID = " + s_ShowingID;
	}


	/**
	 *	Returns error state
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Possible errors encountered
	 */
	public String getError ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::PurchaseEJB:: (getError) " + s_Error );
		return s_Error;
	}


	/**
	 *	Returns page to go to
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Page to go to after processing
	 */
	public String getGoto ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::PurchaseEJB:: (getGoto) " + s_Goto );
		return s_Goto;
	}


	/**
	 *	Returns array of String attributes that need to be set for the
	 *	session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of session attributes to set
	 */
	public String[] getSessionAttributes ( ) throws java.rmi.RemoteException
	{
		System.out.println ( "::PurchaseEJB:: (getSessionAttributes)" );
		return null;
	}


	/**
	 *	Processes stored purchase data.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Contents to display to interface
	 */
    public String process ( ) throws java.rmi.RemoteException
    {
		System.out.println ( "::PurchaseEJB:: (process) Begin." );
        Connection conn = null;
        Statement stmt = null;
        ResultSet result = null;

		System.out.println ( "::PurchaseEJB:: (process) Starting Datasource access." );

        try
        {
			ResourceBundle rb = ResourceBundle.getBundle ( "LocalStrings", Locale.getDefault ( ) );

            conn = DBUtil.getDBConnection ( );
            stmt = conn.createStatement ( );

            result = stmt.executeQuery ( this.getOrderQuery ( ) );
			System.out.println ( "::PurchaseEJB:: (process) Order query executed." );
            result.last ( );
            String into = new String ( "" + ( result.getRow ( ) + 1 ) );
            result = stmt.executeQuery ( this.getOrderInsert ( into ) );
			System.out.println ( "::PurchaseEJB:: (process) Order insert executed." );
            result = stmt.executeQuery ( this.getShowingUpdate ( ) );
			System.out.println ( "::PurchaseEJB:: (process) Showing update executed." );
            result = stmt.executeQuery ( this.getShowingQuery ( ) );
			System.out.println ( "::PurchaseEJB:: (process) Showing query executed." );
     		ResultSetMetaData rsmd = result.getMetaData();
     		int cols = rsmd.getColumnCount();
            result.next ( );

			String delimiter = "__";
            s_ShowingRow = "";
            for ( int i = 1; i <= cols; i++ )
            	s_ShowingRow = s_ShowingRow + result.getString ( i ) + delimiter;

			return this.getFormattedTable ( );
        }
        catch ( Exception e )
        {
            //e.printStackTrace ( );
			System.out.println ( "::PurchaseEJB:: (process) Undefined error." );
			s_Error = "Undefined error";

        }
		finally
		{
			DBUtil.closeConnection ( conn );
			DBUtil.closeStatement ( stmt );
			DBUtil.closeResultSet ( result );
		}

		//System.out.println ( "::PurchaseEJB:: (process) Undefined error." );
		//s_Error = "Undefined error";

        return s_Error;
	}


    /**
     * Sets the session context. Required by EJB spec.
     * @param ctx A SessionContext object.
     */
    public void setSessionContext(javax.ejb.SessionContext ctx) {
        m_ctx = ctx;
    }

    /**
     * Creates a bean. Required by EJB spec.
     * @exception throws CreateException.
     */
    public void ejbCreate() throws java.rmi.RemoteException, javax.ejb.CreateException {
        System.out.println("ejbCreate() on obj " + this);
    }

    /**
     * Removes the bean. Required by EJB spec.
     */
    public void ejbRemove() {
        System.out.println("ejbRemove() on obj " + this);
    }

    /**
     * Loads the state of the bean from secondary storage. Required by EJB spec.
     */
    public void ejbActivate() {
        System.out.println("ejbActivate() on obj " + this);
    }

    /**
     * Serializes the state of the bean to secondary storage. Required by EJB spec.
     */
    public void ejbPassivate() {
        System.out.println("ejbPassivate() on obj " + this);
    }


	/**
	 *	Returns formatted date string
	 *
	 *	@param	date	Date string in yyyy-mm-dd format
	 *	@returns Date in Month Date, Year format
	 */
	private String getFormattedDate ( String date )
	{
		System.out.println ( "::PurchaseEJB:: (getFormattedDate)" );
		String year = date.substring ( 0, 4 );
		int mnum = ( new Integer ( date.substring ( 5, 7 ) ) ).intValue ( );
		String day = date.substring ( 8, 10 );
		String month = ( mnum == 1 ? "January" :
						 mnum == 2 ? "February" :
						 mnum == 3 ? "March" :
						 mnum == 4 ? "April" :
						 mnum == 5 ? "May" :
						 mnum == 6 ? "June" :
						 mnum == 7 ? "July" :
						 mnum == 8 ? "August" :
						 mnum == 9 ? "September" :
						 mnum == 10 ? "October" :
						 mnum == 11 ? "November" :
						 "December" );

		return month + " " + day + ", " + year;
	}


	/**
	 *	Returns formatted movie table string
	 *
	 *	@param	title	Movie title string
	 *	@param	date	Date string in yyyy-mm-dd format
	 *	@param	time	Time in hh:mm:ss format
	 *	@returns HTML code for movie table
	 */
	private String getFormattedMovie ( String title, String date, String time )
	{
		System.out.println ( "::PurchaseEJB:: (getFormattedMovie)" );
		String retString = new String ( );
		retString = "";

		retString = retString + "<br><table align = center class = MenuTable width = 600 cellpadding = 3>";
		retString = retString + "<tr>";
		retString = retString + "<td class = TheaterName align = center>";
		retString = retString + title;
		retString = retString + "</td>";
		retString = retString + "</tr>";
		retString = retString + "<tr>";
		retString = retString + "<td class = MovieTime align = center>";
		retString = retString + this.getFormattedDate ( date );
		retString = retString + " ";
		retString = retString + this.getFormattedTime ( time );
		retString = retString + "</td>";
		retString = retString + "</tr>";

		return retString;
	}


	/**
	 *	Returns formatted movie table string
	 *
	 *	@param	length	Movie length
	 *	@param	room	Room name
	 *	@param	theater	Theater name
	 *	@returns HTML code for theater table
	 */
	private String getFormattedTheater ( String length, String room, String theater )
	{
		System.out.println ( "::PurchaseEJB:: (getFormattedTheater)" );
		int min = ( new Integer ( length ) ).intValue ( );
		int hour = 0;
		while ( min >= 60 )
		{
			++hour;
			min -= 60;
		}

		return 	"<tr>" +
				"<td class = MovieTime align = center>" +
				"Playing in " + room + " at " + theater +
				"</td>" +
				"</tr>" +
				"<tr>" +
				"<td class = MovieTime align = center>" +
				hour + "h " + min + "m" +
				"</td>" +
				"</tr>" +
				"</table><br>";
	}


	/**
	 *	Returns formatted time string
	 *
	 *	@param	time	Time in hh:mm:ss format
	 *	@returns Time in 12-hour HH:MM am/pm format
	 */
	private String getFormattedTime ( String time )
	{
		System.out.println ( "::PurchaseEJB:: (getFormattedTime)" );
		int hour = ( new Integer ( time.substring ( 0, 2 ) ).intValue ( ) );
		if ( hour >= 12 )
		{
			if ( hour > 12 )
				hour -= 12;
			return "" + ( hour < 10 ? "0" : "" ) + hour + time.substring ( 2, 5 ) + "pm";
		}
		else
		{
			if ( hour == 0 )
				hour = 12;
			return "" + ( hour < 10 ? "0" : "" ) + hour + time.substring ( 2, 5 ) + "am";
		}
	}


	/**
	 *	Returns simple confirmation acknowledgment
	 *
	 *	@param	time	Time in hh:mm:ss format
	 *	@returns Time in 12-hour HH:MM am/pm format
	 */
	private String getConfirmation ( )
	{
		System.out.println ( "::PurchaseEJB:: (getConfirmation)" );
		return "<br><center>Thank you for your purchase.</center>";
	}


	/**
	 *	Returns full page html code for purchase confirmation
	 *
	 *	@returns HTML code for page
	 */
	private String getFormattedTable ( )
	{
		System.out.println ( "::PurchaseEJB:: (getFormattedTable)" );
		String[] Showing = s_ShowingRow.split ( "__" );

		return	this.getFormattedMovie ( Showing[2], Showing[4], Showing[5] ) +
				this.getFormattedTheater ( Showing[3], Showing[1], Showing[0] ) +
				this.getConfirmation ( );
	}
}
