/**
 * Copyright  2002 Sun Microsystems, Inc. All rights reserved.
 */

package edu.cmu.fantango.ejb;

/**
 * Remote interface for the RegisterEJB. The remote interface defines all possible
 * business methods for the bean. These are the methods going tobe invoked remotely
 * by clients, once they have a reference to the remote interface.
 *
 * Clients (MovieServlet, in this case), generally take the help of JNDI to lookup
 * the bean's home interface (RegisterHome, in this case) and then use the home interface
 * to obtain references to the bean's remote interface (Register, in this case).
 *
 */
public interface Showing extends javax.ejb.EJBObject
{
	/**
	 *	Initializes Showing class to store and process
	 *	showing information.
	 *
	 *	@throws java.rmi.RemoteException
	 */
    public void Init ( ) throws java.rmi.RemoteException;

	/**
	 *	Initializes parameters from forms (via request)
	 *
	 *	@param	params Values to assign to needed parameters
	 *	@throws java.rmi.RemoteException
	 */
    public void InitParams ( String[] params ) throws java.rmi.RemoteException;

	/**
	 *	Initializes attributes from request.
	 *
	 *	@param	attribs Values to assign to needed attributes
	 *	@throws java.rmi.RemoteException
	 */
    public void InitAttribs ( String[] attribs ) throws java.rmi.RemoteException;

	/**
	 *	Processes stored showing data.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Contents to display to interface
	 */
    public String process ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns error state
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Possible errors encountered
	 */
	public String getError ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns page to go to
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Page to go to after processing
	 */
	public String getGoto ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns array of String parameters for the servlet to get
	 *	from request form fields.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of parameter names
	 */
	public String[] getParameters ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns array of String attributes for the servlet to get
	 *	from request.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of attribute names
	 */
	public String[] getAttributes ( ) throws java.rmi.RemoteException;

	/**
	 *	Returns array of String attributes that need to be set for the
	 *	session.
	 *
	 *	@throws java.rmi.RemoteException
	 *	@returns Array of strings of session attributes to set
	 */
	public String[] getSessionAttributes ( ) throws java.rmi.RemoteException;
}
