/**
 * EJBFactory.java
 */

package edu.cmu.fantango.servlet;

import java.util.*;
import javax.naming.*;

/**
 * This class is a utility to get a EJB object from JNDI.
 */
public class EJBFactory
{
    /**
     * static method to access a JNDI initial context with EJB objects.
     */
    protected final static int TIMEOUT = 5;
    protected final static String DEFAULT_SERVER = "devilbunny";
    protected static String CURRENT_SERVER = DEFAULT_SERVER;

    public static InitialContext getInitialContext() throws NamingException
    {
        Properties p = new Properties();
        p.put(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
        p.put(Context.URL_PKG_PREFIXES, "jboss.naming:org.jnp.interfaces");
        p.put(Context.PROVIDER_URL, "jnp://"+ CURRENT_SERVER +".ece.cmu.edu:1099");
        return new InitialContext(p);
        //may remove these if we find a way to let servlet know where jndi.properties are.
        //we also need to put client/*.jar file to appropriate lib directories of tomcat
        //container we are using. Should we use rb. to get provider URL?
    }



    /**
     * returns a EJBHome object from its name.
     */
    public static Object getEJBHome(String name)
    {
        InitialContext context = null;
        Object ref = null;
        System.out.println("getEJB(): retrieving home object [" + name + "]");
        try {
            context = getInitialContext();
            ref = context.lookup(name);
        } catch (NamingException e) {
	    ref = failover(name);
        }	
        return ref;
    }

    /**
     * returns next available refernce.
     */
    protected static Object failover(String name)
    {	
	int timeout = 0;
	Object ref = null;
	
	while(timeout <= TIMEOUT){
	    InitialContext context = null;
	    RepMan repman = new RepMan();
	    List list = repman.getList();
	    Iterator i = list.iterator();
	    
	    //repman.checkAndKill(CURRENT_SERVER);
	    
	    while (i.hasNext()) {
		Command cmd = (Command)i.next();
		String machine = cmd.getMachine();
		CURRENT_SERVER = machine;
			
		try{
		    context = getInitialContext();
		    ref = context.lookup(name);
		    System.out.println("failover(): failing over to [" + machine + "]");
		    return ref;
		} catch (NamingException ne) {
		    System.out.println("failover(): could not bind to [" + machine + "] ");
		}
		       
	    }
	    
	    try {
		System.out.println("failover(): wait " + TIMEOUT + " seconds to retry...");
		Thread.sleep(TIMEOUT * 1000);
		timeout++;
	    } catch (Exception te){
		te.printStackTrace();
	    }
	}
	System.out.println("FanTango Server is currently unavailable... please visit later");
	return ref;
    }	


}  // end of class
